using System;
using System.Collections;
using System.ComponentModel;

namespace BUGLists
{
	#region csCustomer class
	/// <summary>
	/// Summary description for csCustomer.
	/// </summary>
	public class csCustomer: ICustomer, IComparable, IEditableObject
	{
		#region Private instance fields
		private IList _owner;
		private bool _newCustomer;
		private csCustomer _backupData;
		private bool _editing;
		#endregion

		#region Instance variables for properties
		private string _title;
		private string _firstName;
		private string _lastName;
		#endregion

		#region Constructor
		public csCustomer( IList p_owner )
		{
			if ( p_owner != null )
			{
				_owner = p_owner;
			}
			_newCustomer = true;
		}
		#endregion

		#region Default string representation
		public override string ToString()
		{
			return ( ( ICustomer )this ).FullName;
		}
		#endregion

		#region ICustomer members
		// Note - they're public so grids can display them
		public string FirstName
		{
			get { return _firstName; }
			set	{ _firstName = value; }
		}

		public string LastName
		{
			get	{ return _lastName;	}
			set	{ _lastName = value; }
		}

		public string Title
		{
			get	{ return _title; }
			set	{ _title = value; }
		}

		public string FullName
		{
			get
			{
				// Put the Title, First name & Last name together
				return _title + ' ' + _firstName + ' ' + _lastName;
			}
		}
		#endregion

		#region IComparable members
		int IComparable.CompareTo( object p_object )
		{
			if ( p_object is csCustomer )
			{
				return this.CompareTo( ( csCustomer ) p_object );
			}
			else
			{
				throw new ArgumentException( "object is not a Customer" );
			}
		}
		#endregion

		#region Typed CompareTo
		public int CompareTo( ICustomer p_customer )
		{
			string l_mySortName = _lastName + _firstName + _title;
			string l_objectsSortName = ( ( ICustomer ) p_customer ).LastName +
				( ( ICustomer ) p_customer ).FirstName +
				( ( ICustomer ) p_customer ).Title;
			return l_mySortName.CompareTo( l_objectsSortName );
		}
		#endregion

		#region Utility functions
		private bool IsChanged()
		{
			return ( _title != _backupData._title ) || 
				   ( _firstName != _backupData._firstName ) || 
				   ( _lastName != _backupData._lastName );
		}
		#endregion

		#region IEditableObject members
		void IEditableObject.BeginEdit()
		{
			if ( ! _editing )
			{
				if ( _backupData == null )
				{
					_backupData = new csCustomer( null );
				}
				_backupData._title = _title;
				_backupData._firstName = _firstName;
				_backupData._lastName = _lastName;
				_editing = true;
			}
		}

		void IEditableObject.CancelEdit()
		{
			if ( _editing )
			{
				if ( _backupData != null )
				{
					_title = _backupData._title;
					_firstName = _backupData._firstName;
					_lastName = _backupData._lastName;
				}
				if ( _newCustomer )
				{
					_owner.Remove( this );
				}
				_editing = false;
			}
		}

		void IEditableObject.EndEdit()
		{
			_newCustomer = false;
			_editing = false;
			if ( IsChanged() )
			{
				// Trigger the OnSetComplete event to update controls
				int l_position = _owner.IndexOf( this );
				_owner[ l_position ] = this;
			}
		}
		#endregion

	}
	#endregion

	#region Comparers
	public class csCompareCustomerByTitle: IComparer
	{
		int IComparer.Compare( object p_customer1, object p_customer2 )
		{
			if ( ( p_customer1 != null ) && ( p_customer1 is ICustomer ) && ( p_customer2 != null ) && ( p_customer2 is ICustomer ) )
			{
				return ( ( ICustomer ) p_customer1 ).Title.CompareTo( ( ( ICustomer ) p_customer2 ).Title );
			}
			else
			{
				throw new ArgumentException( "Incorrect arguments passed to Compare" );
			}
		}
	}

	public class csCompareCustomerByFirstName: IComparer
	{
		int IComparer.Compare( object p_customer1, object p_customer2 )
		{
			if ( ( p_customer1 != null ) && ( p_customer1 is ICustomer ) && ( p_customer2 != null ) && ( p_customer2 is ICustomer ) )
			{
				return ( ( ICustomer ) p_customer1 ).FirstName.CompareTo( ( ( ICustomer ) p_customer2 ).FirstName );
			}
			else
			{
				throw new ArgumentException( "Incorrect arguments passed to Compare" );
			}
		}
	}

	public class csCompareCustomerByLastName: IComparer
	{
		int IComparer.Compare( object p_customer1, object p_customer2 )
		{
			if ( ( p_customer1 != null ) && ( p_customer1 is ICustomer ) && ( p_customer2 != null ) && ( p_customer2 is ICustomer ) )
			{
				return ( ( ICustomer ) p_customer1 ).LastName.CompareTo( ( ( ICustomer ) p_customer2 ).LastName );
			}
			else
			{
				throw new ArgumentException( "Incorrect arguments passed to Compare" );
			}
		}
	}

	public class csCompareCustomer: IComparer
	{
		int IComparer.Compare( object p_customer1, object p_customer2 )
		{
			if ( ( p_customer1 != null ) && ( p_customer1 is ICustomer ) && ( p_customer2 != null ) && ( p_customer2 is ICustomer ) )
			{
				ICustomer l_cust1 = ( ICustomer ) p_customer1;
				string l_name1 = l_cust1.LastName + l_cust1.FirstName + l_cust1.Title;
				ICustomer l_cust2 = ( ICustomer ) p_customer2;
				string l_name2 = l_cust2.LastName + l_cust2.FirstName + l_cust2.Title;
				return l_name1.CompareTo( l_name2 );
			}
			else
			{
				throw new ArgumentException( "Incorrect arguments passed to Compare" );
			}
		}
	}
	#endregion
																							 }
